@@ -8,15 +8,21 @@ Time::Format - Easy-to-use date/time formatting.
 
 =head1 VERSION
 
-This documentation describes version 1.00 of Time::Format.pm, September 22, 2004.
+This documentation describes version 1.01 of Time::Format.pm, November 17, 2005.
 
 =cut
 
 use strict;
 package Time::Format;
 
-$Time::Format::VERSION  = '1.00';
-%Time::Format::XSCOMPAT = map {$_ => 1} qw/0.13 1.00/;   # Compatible with these versions of Time::Format_XS.
+$Time::Format::VERSION  = '1.01';
+%Time::Format::XSCOMPAT = map {$_ => 1} qw(1.01);   # Compatible with these versions of Time::Format_XS.
+
+sub _croak
+{
+    require Carp;
+    goto &Carp::croak;
+}
 
 my $load_perlonly = 0;
 
@@ -43,13 +49,44 @@ if (!$load_perlonly)
         }
     }
 
+    # Okay to use the XS version?  Great.  Wrap it.
     if (!$load_perlonly)
     {
         *time_format = sub
         {
             my $fmt = shift;
-            my $time = (@_  &&  $_[0] ne 'time')? shift : _have('Time::HiRes')? Time::HiRes::time() : time;
-            Time::Format_XS::time_format($fmt, $time);
+            my $time;
+
+            # If there's an argument, it should be a DateTime object
+            # or a numeric (epoch-based) time number.
+            # If it's 'time', the user goofed; but we know what they meant.
+            # Update, 2005/11/16: DateTime stringification (YYYY-MM-DDTHH:MM:SS)
+            # and Date::Manip stringification (YYYYMMDDHH:MM:SS) are supported.
+            my $arg = @_? shift : 'time';
+            if (UNIVERSAL::isa($arg, 'DateTime'))
+            {
+                $time = $arg->hires_epoch()
+            }
+            elsif ($arg eq 'time'  ||  $arg eq q{})
+            {
+                $time = _have('Time::HiRes')? Time::HiRes::time() : time;
+            }
+            elsif ($arg =~ /(\d{10}:\d{2}:\d{2})/)    # Date::Manip stringification
+            {
+                _croak qq{Unrecognized time value "$arg"} unless _have ('Date::Manip');
+                $time = Date::Manip::UnixDate($1, '%s');
+            }
+            elsif ($arg =~ /(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(?:[.,](\d+))?/)  # DateTime stringification
+            {
+                _croak qq{Unrecognized time value "$arg"} unless _have ('DateTime');
+                my $nano = defined $7? ($7 . ('0' x (9-length($7)))) : 0;
+                my $dt = DateTime->new(year=>$1, month=>$2, day=>$3,
+                                       hour=>$4, minute=>$5, second =>$6, nanosecond => $nano);
+                $time = $dt->hires_epoch();
+            }
+
+            # Format it.
+            return Time::Format_XS::time_format($fmt, $time);
         }
     }
 }
@@ -59,15 +96,15 @@ if ($load_perlonly)
     # Time::Format_XS not installed, or version mismatch, or NOXS was set.
     # The perl routines will need to be loaded.
     # But defer this until someone actually calls time_format().
-    *time_format_perlonly = sub
+    *time_format = sub
     {
         local $^W = 0;    # disable warning about subroutine redefined
         local $/ = undef;
         eval <DATA>;
         die if $@;
-        goto &time_format_perlonly;
+        *time_format = \&time_format_perlonly;
+        goto &time_format;
     };
-    *time_format = sub { goto &time_format_perlonly };
     undef $Time::Format_XS::VERSION;    # Indicate that XS version is not available.
 }
 
@@ -102,7 +139,7 @@ sub import
     {
         my $s = @badsym>1? 's'   : '';
         my $v = @badsym>1? 'are' : 'is';
-        die ("The symbol$s ", join(', ', @badsym), " $v not exported by Time::Format at $file line $line.\n");
+        _croak ("The symbol$s ", join(', ', @badsym), " $v not exported by Time::Format at $file line $line.\n");
     }
 
     no strict 'refs';
@@ -159,7 +196,7 @@ use subs qw(
 *STORE = *EXISTS = *CLEAR = *FIRSTKEY = *NEXTKEY = sub
 {
     my ($pkg,$file,$line) = caller;
-    die "Invalid call to Time::Format internal function at $file line $line.";
+    _croak "Invalid call to Time::Format internal function at $file line $line.";
 };
 
 
@@ -237,6 +274,7 @@ my $cache_size=0;           # Number of keys in %loc_cache
 my $cache_size_limit = 256; # Max number of times to cache
 
 # Internal function to initialize locale info.
+# Returns true if the locale changed.
 sub setup_locale
 {
     # Do nothing if locale has not changed since %names was set up.
@@ -289,60 +327,141 @@ sub setup_locale
     %loc_cache = ();          # locale changes are rare.  Clear out cache.
     $cache_size = 0;
     $locale = $locale_in_use;
+
+    return 1;
 }
 
+# Types of time values we can handle:
+my $NUMERIC_TIME      = \&decipher_epoch;
+my $DATETIME_OBJECT   = \&decipher_DateTime_object;
+my $DATETIME_STRING   = \&decipher_DateTime_string;
+my $DATEMANIP_STRING  = \&decipher_DateManip_string;
+
+# What kind of argument was passed to time_format?
+# Returns (type, time, cache_time_key, milliseconds, microseconds)
+sub _classify_time
+{
+    my $timeval = shift;
+    $timeval = 'time' if !defined $timeval;
+
+    my $frac;  # Fractional seconds, if any
+    my $cache_value;    # 1/20 of 1 cent
+    my $time_type;
+
+    # DateTime object?
+    if (UNIVERSAL::isa($timeval, 'DateTime'))
+    {
+        $cache_value = "$timeval";    # stringify
+        $frac        = $timeval->nanosecond() / 1e9;
+        $time_type   = $DATETIME_OBJECT;
+    }
+    # Numeric time?
+    elsif ($timeval =~ /^\s* (  (\d+) (?:[.,](\d+))?  )  $/x)
+    {
+        $timeval     = $1;
+        $cache_value = $2;
+        $frac        = $3? '0.' . $3 : 0;
+        $time_type   = $NUMERIC_TIME;
+    }
+    # Not set, or set to 'time' string
+    elsif ($timeval eq 'time'  ||  $timeval eq q{})
+    {
+        # Get numeric time
+        $timeval     = _have('Time::HiRes')? Time::HiRes::time() : time;
+        $cache_value = int $timeval;
+        $frac        = $cache_value - $timeval;
+        $time_type   = $NUMERIC_TIME;
+    }
+    # Stringified DateTime object
+    # Except we make it more flexible by allowing the date OR the time to be specfied
+    elsif ($timeval =~ /\A(
+                          (?:
+                            \d{4} - \d{2} - \d{2}          # year-month-day
+                            (?!\d) )?                      # ymd is optional, but next must not be digit
+                          (?:  (?<=\d)  [T_ ]  (?=\d) )?   # separator: T or _ or space, but only if ymd and hms both present
+                        )                                  # End of $1: YMD and separator
+                          (?:  (?<!\d)                     # hms is optional, but must be preceded by digit
+                        (
+                            \d{2} : \d{2} : \d{2}          # hour:minute:second
+                        )                                  # End of $2: HMS
+                            (?: [,.] (\d+))?               # optional fraction
+                          )?                               # end of optional (HMS.fraction)
+                        \z
+                     /x)
+    {
+        $cache_value = ($1 || q{}) . ($2 || q{});
+        $frac        = $3? '0.' . $3 : 0;
+        $time_type   = $DATETIME_STRING;
+    }
+    # Date::Manip stringification
+    elsif ($timeval =~ /((\d{10}:\d{2}:\d{2})(?:[.,](\d+))?)/x)
+    {
+        $timeval     = $1;
+        $cache_value = $2;
+        $frac        = $3? '0.' . $3 : 0;
+        $time_type   = $DATEMANIP_STRING;
+    }
+    else
+    {
+        # User passed us something we don't know how to handle.
+        _croak qq{Unrecognized time value: "$timeval"};
+    }
+    # We messed up.
+    die qq{Illegal time type "$time_type"; programming error in Time::Format. Contact author.}
+        if !defined &$time_type;
+
+    # Calculate millisecond, microsecond from fraction
+    # msec and usec are TRUNCATED, not ROUNDED, because rounding up
+    # to the next higher second would be a nightmare.
+    my $msec = sprintf '%03d', int (    1_000 * $frac);
+    my $usec = sprintf '%06d', int (1_000_000 * $frac);
+
+    return ($time_type, $timeval, $cache_value, $msec, $usec);
+}
 
 # Helper function -- returns localtime() hashref
 sub _loctime
 {
-    my $time;
-    if (@_  &&  $_[0] ne 'time')
-    {
-        $time = shift;
-    }
-    else
-    {
-        $time = _have('Time::HiRes')? Time::HiRes::time() : time();
-    }
-    my $it = int $time;
-    my $msec = sprintf '%03d', int (    1_000 * ($time - $it));
-    my $usec = sprintf '%06d', int (1_000_000 * ($time - $it));
-
-    setup_locale;
+    my ($decipher, $time, $cachekey, $msec, $usec) = _classify_time(@_);
+    my $locale_changed = setup_locale;
 
     # Cached, because I expect this'll be called on the same time values frequently.
-    if (exists $loc_cache{$it})
+    die "Programming error: undefined cache value. Contact Time::Format author."
+        if !defined $cachekey;
+
+    # If locale has changed, can't use the cached value.
+    if (!$locale_changed  &&  exists $loc_cache{$cachekey})
     {
-        my $h = $loc_cache{$it};
-        @$h{qw/mmm uuuuuu/} = ($msec, $usec);
+        my $h = $loc_cache{$cachekey};
+        ($h->{mmm}, $h->{uuuuuu}) = ($msec, $usec);
         return $h;
     }
 
-    my @t = localtime $it;
-    my ($h,$d,$mx,$wx) = @t[2,3,4,6];    # Month, hour, Month, Weekday indexes.
-    $t[4]++;
-    my $h12 = $h>12? $h-12 : ($h || 12);
-    my $tz = _have('POSIX')? POSIX::strftime('%Z',@t) : '';
+    # Hour-12, time zone, localtime parts, deciphered from input
+    my ($h12, $tz, @time_parts) = $decipher->($time);
 
     # Populate a whole mess o' data elements
     my %th;
+    my $m0 = $time_parts[4] - 1;   # zero-based month
+    $tz ||= _have('POSIX')? POSIX::strftime('%Z',@time_parts) : q{};
 
     # NOTE: When adding new codes, be wary of adding any that interfere
     # with the user's ability to use the words "at", "on", or "of" literally.
 
     # year, hour(12), month, day, hour, minute, second, millisecond, microsecond, time zone
-    @th{qw[yyyy H  m{on}  d  h  m{in}  s  mmm uuuuuu tz]} = (    $t[5]+1900, $h12, @t[4,3,2,1,0], $msec, $usec, $tz);
-    @th{qw[yy  HH mm{on} dd hh mm{in} ss]} = map $_<10?"0$_":$_, $t[5]%100,  $h12, @t[4,3,2,1,0];
-    @th{qw[    ?H ?m{on} ?d ?h ?m{in} ?s]} = map $_<10?" $_":$_,             $h12, @t[4,3,2,1,0];
+    @th{qw[yyyy H  m{on}  d  h  m{in}  s  mmm uuuuuu tz]} = (    $time_parts[5],      $h12, @time_parts[4,3,2,1,0], $msec, $usec, $tz);
+    @th{qw[yy  HH mm{on} dd hh mm{in} ss]} = map $_<10?"0$_":$_, $time_parts[5]%100,  $h12, @time_parts[4,3,2,1,0];
+    @th{qw[    ?H ?m{on} ?d ?h ?m{in} ?s]} = map $_<10?" $_":$_,                      $h12, @time_parts[4,3,2,1,0];
 
     # AM/PM
+    my ($h,$d,$wx) = @time_parts[2,3,6];  # Day, weekday index
     my $a = $h<12? 'a' : 'p';
     $th{am}     = $th{pm}     = $a . 'm';
     $th{'a.m.'} = $th{'p.m.'} = $a . '.m.';
     @th{qw/AM PM A.M. P.M./} = map uc, @th{qw/am pm a.m. p.m./};
 
     $th{$_} = $names{$_}[$wx] for qw/Weekday WEEKDAY weekday Day DAY day/;
-    $th{$_} = $names{$_}[$mx] for qw/Month   MONTH   month   Mon MON mon/;
+    $th{$_} = $names{$_}[$m0] for qw/Month   MONTH   month   Mon MON mon/;
     $th{$_} = $names{$_}[$d]  for qw/th TH/;
 
     # Don't let the time cache grow boundlessly.
@@ -351,9 +470,123 @@ sub _loctime
         $cache_size = 0;
         %loc_cache = ();
     }
-    return $loc_cache{$it} = \%th;
+    return $loc_cache{$cachekey} = \%th;
 }
 
+sub decipher_DateTime_object
+{
+    my $dt = shift;
+
+    my @t = ($dt->hour_12, $dt->time_zone_short_name,
+             $dt->second,  $dt->minute, $dt->hour,
+             $dt->day,     $dt->month,  $dt->year,
+             $dt->day_of_week);
+    $t[-1] = 0 if $t[-1] == 7;   # Convert 1-7 (Mon-Sun) to 0-6 (Sun-Sat).
+
+    return @t;
+}
+
+# 2005-10-31T15:14:39
+sub decipher_DateTime_string
+{
+    my $dts = shift;
+    unless ($dts =~ /\A(?:
+                         (\d{4}) - (\d{2}) - (\d{2})    # year-month-day
+                      (?!\d) )?                         # ymd is optional, but next must not be digit
+                      (?:  (?<=\d)  [T_ ]  (?=\d) )?    # separator: T or _ or space, but only if ymd and hms both present
+                      (?: (?<!\d)                       # hms is optional, but must be preceded by digit
+                         (\d{2}) : (\d{2}) : (\d{2})    # hour:minute:second
+                         (?: [,.] \d+)?                 # optional fraction (ignored in this sub)
+                      )?   \z
+                     /x)
+    {
+        # This "should" never happen, since we checked the format of
+        # the string already.
+        die "Unrecognized DateTime string: probable Time::Format bug";
+    }
+
+    my ($y,$mon,$d,$h,$min,$s) = ($1,$2,$3,$4,$5,$6);
+    $y   ||= 1969;
+    $mon ||=   12;
+    $d   ||=   31;
+    $h   ||=    0;
+    $min ||=    0;
+    $s   ||=    0;
+    my @t = map {$_+0} ($s,$min,$h,$d,$mon,$y);   # +0 is to force numeric (remove leading zeroes)
+    my $h12  = $h == 0? 12
+             : $h > 12? $h - 12
+             :          $h;
+    push @t, _dow($y, $mon, $d);
+    return ($h12, undef, @t);
+}
+
+# 2005103115:14:39
+sub decipher_DateManip_string
+{
+    my $dts = shift;
+    unless ($dts =~ /\A
+                         (\d{4})   (\d{2})   (\d{2})    # year-month-day
+                         (\d{2}) : (\d{2}) : (\d{2})    # hour:minute:second
+                         (?: [,.] \d+)?                 # optional fraction (ignored in this sub)
+                     \z
+                     /x)
+    {
+        # This "should" never happen, since we checked the format of
+        # the string already.
+        die "Unrecognized DateTime string: probable Time::Format bug";
+    }
+
+    my ($y,$mon,$d,$h,$min,$s) = ($1,$2,$3,$4,$5,$6);
+    $y   ||= 1969;
+    $mon ||=   12;
+    $d   ||=   31;
+    $h   ||=    0;
+    $min ||=    0;
+    $s   ||=    0;
+    my @t = map {$_+0} ($s,$min,$h,$d,$mon,$y);   # +0 is to force numeric (remove leading zeroes)
+    my $h12  = $h == 0? 12
+             : $h > 12? $h - 12
+             :          $h;
+    push @t, _dow($y, $mon, $d);
+    return ($h12, undef, @t);
+}
+
+sub decipher_epoch
+{
+    my $time = shift;   # Assumed to be an epoch time integer
+
+    my @t = localtime $time;
+    my $h = $t[2];    # Hour (24), Month index
+    $t[4]++;
+    $t[5] += 1900;
+    my $h12 = $h>12? $h-12 : ($h || 12);
+
+    return ($h12, undef, @t);
+}
+
+#  $int = dow ($year, $month, $day);
+#
+# Returns the day of the week (0=Sunday .. 6=Saturday).  Uses Zeller's
+# congruence, so it is't subject to the unix 2038 limitation.
+#
+#--->     $int = dow ($year, $month, $day);
+sub _dow
+{
+    my ($Y, $M, $D) = @_;
+
+    $M -= 2;
+    if ($M < 1)
+    {
+        $M += 12;
+        $Y--;
+    }
+    my $C = int($Y/100);
+    $Y %= 100;
+
+    return (int((26*$M - 2)/10) + $D + $Y + int($Y/4) + int($C/4) - 2*$C) % 7;
+}
+
+
 # The heart of the module.  Didja ever see so many wicked regexes in a row?
 
 my %disam;    # Disambiguator for 'm' format.
@@ -362,7 +595,7 @@ $disam{$_} = "{in}" foreach qw/h hh ?h H HH ?H s ss ?s/;   # If hour or second i
 sub time_format_perlonly
 {
     my $fmt  = shift;
-    my $time = &_loctime;
+    my $time = _loctime(@_);
 
     # Remove \Q...\E sequences
     my $rc;
@@ -400,7 +633,7 @@ sub time_format_perlonly
                  )
               )/$1$disam{$3}$2/gx;
 
-    # The Big Date/Time Pattern
+    # The Big Date/Time Pattern of Death
     $fmt =~ s/
               (?<!\\)                      # Don't expand something preceded by backslash
               (?=[dDy?hHsaApPMmWwutT])     # Jump to one of these characters
@@ -456,7 +689,7 @@ sub time_format_perlonly
         {
             return $rcode unless index($str, $rcode) >= 0;
         }
-        die "Time::Format cannot process string: no unique characters left.";
+        _croak "Time::Format cannot process string: no unique characters left.";
     }
 
     sub remember
@@ -492,6 +725,13 @@ __END__
  print "Another time is $time{'H:mm am tz', $another_time}\n";
  print "Timestamp: $time{'yyyymmdd.hhmmss.mmm'}\n";
 
+C<%time> also accepts Date::Manip strings and DateTime objects:
+
+ $dm = Date::Manip::ParseDate('last monday');
+ print "Last monday was $time{'Month d, yyyy', $dm}";
+ $dt = DateTime->new (....);
+ print "Here's another date: $time{'m/d/yy', $dt}";
+
  $strftime{$format}
  $strftime{$format, $unixtime}
  $strftime{$format, $sec,$min,$hour, $mday,$mon,$year, $wday,$yday,$isdst}
@@ -501,7 +741,7 @@ __END__
  print "POSIXish: $strftime{'%A, %B %d, %Y'}\n";       # current time
 
  $manip{$format};
- $manip{$format,$when};
+ $manip{$format, $when};
 
  print "Date::Manip: $manip{'%m/%d/%Y'}\n";            # current time
  print "Date::Manip: $manip{'%m/%d/%Y','last Tuesday'}\n";
@@ -677,9 +917,9 @@ removed and are no longer supported.
  $strftime{$format, $unixtime}
  $strftime{$format}
 
-For those who prefer I<strftime(3)>'s weird % formats, or who need
-POSIX compliance, or who need week numbers or other features C<%time>
-does not provide.
+For those who prefer L<strftime|POSIX/strftime>'s weird % formats, or
+who need POSIX compliance, or who need week numbers or other features
+C<%time> does not provide.
 
 =item manip
 
@@ -688,7 +928,7 @@ does not provide.
 
 Provides an interface to the Date::Manip module's C<UnixDate>
 function.  This function is rather slow, but can parse a very wide
-variety of date input.  See the Date::Manip module for details
+variety of date input.  See the >KDate::Manip> module for details
 about the inputs accepted.
 
 If you want to use the C<%time> codes, but need the input flexibility
@@ -910,9 +1150,9 @@ under the same terms as Perl itself.
 -----BEGIN PGP SIGNATURE-----
 Version: GnuPG v1.2.4 (Cygwin)
 
-iD8DBQFBVEQNY96i4h5M0egRAv/EAKDUzu2gQULXcgAH4aADI7SRwGTAAgCfSyqP
-nlVmbT8Eadkk7usWBJ5tcI8=
-=8qhi
+iD8DBQFBVFR5Y96i4h5M0egRAjjXAJ9W80HgtzB7y3jDcPjRTPtmu32RYgCghTUX
+oGcjgc+O6/vb4Q9mlaQm2bg=
+=o0rP
 -----END PGP SIGNATURE-----
 
 =end gpg
